"use strict";
/**
 *  lambda-lambda-lambda/cli
 *  Command-line tool to create a new L³ application.
 *
 *  Copyright 2023-2024, Marc S. Brooks (https://mbrooks.info)
 *  Licensed under the MIT license:
 *  http://www.opensource.org/licenses/mit-license.php
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addPackage = addPackage;
exports.listPackages = listPackages;
const change_case_1 = require("change-case");
const node_fetch_1 = __importDefault(require("node-fetch"));
const fs = __importStar(require("fs"));
// Github repository URLs.
const REPO_CONTENT_URL = 'https://raw.githubusercontent.com/lambda-lambda-lambda/middleware/master';
const REPO_PLUGIN_URL = 'https://api.github.com/repos/lambda-lambda-lambda/middleware/contents/plugins';
const REPO_PUBLIC_URL = 'https://github.com/lambda-lambda-lambda/middleware/tree/master';
/**
 * Install remote middleware
 */
async function addPackage(name) {
    const cwd = process.cwd();
    if (!isAppRoot(cwd)) {
        throw new Error('Command must be executed in APP_ROOT');
    }
    const fileName = (0, change_case_1.pascalCase)(name);
    const outFile = `${cwd}/src/middleware/${fileName}.js`;
    if (fs.existsSync(outFile)) {
        throw new Error('Middleware already exists');
    }
    const content = await getPackage(fileName);
    if (content) {
        fs.writeFileSync(outFile, content, 'utf8');
        return `${REPO_PUBLIC_URL}/plugins/${name}/README.md`;
    }
}
;
/**
 * Request plugin list from the package repo.
 */
function listPackages() {
    return (0, node_fetch_1.default)(REPO_PLUGIN_URL)
        // Handle errors.
        .then(response => {
        const { status } = response;
        if ([404, 500].includes(status)) {
            throw new Error('Package list not available');
        }
        return response;
    })
        // Parse response.
        .then(response => response.text())
        .then(body => JSON.parse(body));
}
/**
 * Fetch remote file; return text as string.
 */
function getPackage(name) {
    return (0, node_fetch_1.default)(`${REPO_CONTENT_URL}/plugins/${name}/src/plugin.js`)
        // Handle errors.
        .then(response => {
        const { status } = response;
        if ([404, 500].includes(status)) {
            throw new Error(`Plugin name '${name}' not found`);
        }
        return response;
    })
        // Parse response.
        .then(response => response.text());
}
/**
 * Check for app root path.
 */
function isAppRoot(path) {
    return !!fs.existsSync(`${path}/src/app.js`);
}
//# sourceMappingURL=installer.js.map